package turingmachine;

import java.awt.*;
import java.awt.geom.*;

import javax.swing.*;

/**
 * A simple arrow component.
 */
class CTMComponentArrow
	extends JComponent
{
  /**
   * The start point of the arrow in view coordinates.
   */
  private CTMPoint mStart;

  /**
   * The end point of the arrow in view coordinates.
   */
  private CTMPoint mEnd;

  /**
   * Whether or not the arrow is currently being used to make a connection.
   */
  private boolean mIsDragging;

  /**
   * Default constructor.
   */
  public CTMComponentArrow()
  {
	// tell swing that this component doesn't draw its complete area
	setOpaque( false );

	mStart = new CTMPoint();
	mEnd = new CTMPoint();
	mIsDragging = false;

	// default arrow colour
	setForeground( Color.black );
	setFocusable( false );
  }

  /**
   * Set whether or not the arrow is currently being used to make a
   * connection.
   *
   * @param aIsDragging true if dragging else false.
   */
  public void setDragging( boolean aIsDragging )
  {
	mIsDragging = aIsDragging;
  }

  /**
   * Get whether or not the arrow is currently being used to make a
   * connection.
   *
   * @return true if it is currently making a connection else false.
   */
  public boolean isDragging()
  {
	return mIsDragging;
  }

  /**
   * Set the arrows start and end positions in parent coordinates.
   *
   * @param aStart the start point for the arrow.
   * @param aEnd the end point for the arrow.
   */
  public void set( Point2D aStart, Point2D aEnd )
  {
	mStart.setLocation( aStart );
	mEnd.setLocation( aEnd );

	Rectangle r = new Rectangle();
	r.setFrameFromDiagonal( aStart, aEnd );

	// resize the component bounds just big enough to hold the arrow
	setBounds( r.x, r.y, r.width+1, r.height+1 );
  }

  /**
   * Paint the arrow.
   *
   * @param g the Graphics context to paint the arrow on.
   */
  public void paint( Graphics g )
  {
	Graphics2D g2 = ( Graphics2D ) g;

	g2.drawLine( ( int ) ( mStart.x-getX() ), ( int ) ( mStart.y-getY() ),
		( int ) ( mEnd.x-getX() ), ( int ) ( mEnd.y-getY() ) );
  }
}
