package turingmachine;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * A box containing any relevant comments for a diagram.
 */
public class CTMComponentComment
	extends CTMComponent
{
  /**
   * Text are that displays the comments.
   */
  private MyTextArea textArea;

  /**
   * Inset size on the comment component.
   */
  private final static int borderInsetSize = 4;

  /**
   * Whether before a mouse click the mouse cursor was over the relevant edge
   * of the component.
   */
  private boolean N, E, S, W;

  /**
   * Last drag position used for resizing component.
   */
  private Point lastDragPos = null;

  /**
   * Constructor.
   *
   * @param dataComment The corresponding data instance for this component.
   */
  public CTMComponentComment( CTMDataComment dataComment )
  {
	super( dataComment );

	setLayout( new BorderLayout() );
	setBorder( BorderFactory.createEmptyBorder( borderInsetSize,
		borderInsetSize, borderInsetSize, borderInsetSize ) );

	setSize( dataComment.getSize() );
	setLocation( dataComment.getLocation() );
	setOpaque( false );
	setVisible( true );

	textArea = new MyTextArea();
	textArea.setOpaque( false );
	textArea.setWrapStyleWord( true );
	textArea.setLineWrap( true );
	add( textArea, BorderLayout.CENTER );

	textArea.addKeyListener( new KeyAdapter()
	{
	  // update comment Data on comment text change.
	  public void keyReleased( java.awt.event.KeyEvent e )
	  {
		CTMDataComment dc = ( CTMDataComment ) getData();
		dc.setComment( textArea.getText() );
		// getDocument().notifyObservers();
	  }
	} );

	textArea.addFocusListener( new FocusListener()
	{
	  public void focusGained( FocusEvent e )
	  {}

	  public void focusLost( FocusEvent e )
	  {
		// need to repaint as border is different depending on focal
		// state and content
		textArea.setBubbleEvents( false );
		repaint();
	  }
	} );

	addFocusListener( new FocusListener()
	{
	  public void focusGained( FocusEvent e )
	  {
		repaint();
	  }

	  public void focusLost( FocusEvent e )
	  {
		repaint();
	  }
	} );

	addMouseMotionListener( new MouseMotionAdapter()
	{
	  public void mouseMoved( MouseEvent e )
	  {
		this_mouseMoved( e );
	  }

	  public void mouseDragged( MouseEvent e )
	  {
		this_mouseDragged( e );
	  }
	} );

	addMouseListener( new MouseAdapter()
	{
	  public void mouseReleased( MouseEvent e )
	  {
		lastDragPos = null;
		( ( CTMDataComment ) getData() ).setBounds( getBounds() );
		//getDocument().notifyObservers();
	  }

	  public void mousePressed( MouseEvent e )
	  {
		requestFocus();

		// if right click show context menu
		if ( ( e.getModifiers()&MouseEvent.BUTTON1_MASK )==0 )
		{
		  // else must be context click, invoke state popup
		  JPopupMenu popup = new CTMComponentPopupMenu(
							 CTMComponentComment.this );
		  popup.show( CTMComponentComment.this, e.getX()+2,
			  e.getY()+2 );
		}
		else
		{
		  if ( getViewComponent().getSelectedTool()==CTMFrame.TOOL_OPTION.DELETE )
		  {
			getData().delete();
			//getDocument().notifyObservers();
		  }
		}
	  }

	  public void mouseClicked( MouseEvent e )
	  {
		if ( e.getClickCount()==2 )
		{
		  textArea.setBubbleEvents( true );
		  textArea.requestFocus();
		}
	  }
	} );

	addKeyListener( new KeyAdapter()
	{
	  public void keyPressed( KeyEvent e )
	  {
		if ( e.getKeyCode()==KeyEvent.VK_DELETE
			||e.getKeyCode()==KeyEvent.VK_BACK_SPACE )
		{
		  getData().delete();
		  //getDocument().notifyObservers();
		}
	  }
	} );
  }

  /**
   * Mouse has moved. Check the correct cursor is selected.
   *
   * @param e MouseEvent The mouse event.
   */
  private void this_mouseMoved( MouseEvent e )
  {
	if ( e.getButton()==0 )
	{
	  W = e.getPoint().x<=borderInsetSize;
	  E = e.getPoint().x>=getWidth()-borderInsetSize;
	  N = e.getPoint().y<=borderInsetSize;
	  S = e.getPoint().y>=getHeight()-borderInsetSize;

	  if ( W&&N )
	  {
		setCursor( Cursor.getPredefinedCursor( Cursor.NW_RESIZE_CURSOR ) );
	  }
	  else
	  if ( E&&N )
	  {
		setCursor( Cursor
			.getPredefinedCursor( Cursor.NE_RESIZE_CURSOR ) );
	  }
	  else
	  if ( W&&S )
	  {
		setCursor( Cursor
			.getPredefinedCursor( Cursor.SW_RESIZE_CURSOR ) );
	  }
	  else
	  if ( E&&S )
	  {
		setCursor( Cursor
			.getPredefinedCursor( Cursor.SE_RESIZE_CURSOR ) );
	  }
	  else
	  if ( W )
	  {
		setCursor( Cursor
			.getPredefinedCursor( Cursor.W_RESIZE_CURSOR ) );
	  }
	  else
	  if ( E )
	  {
		setCursor( Cursor
			.getPredefinedCursor( Cursor.E_RESIZE_CURSOR ) );
	  }
	  else
	  if ( N )
	  {
		setCursor( Cursor
			.getPredefinedCursor( Cursor.N_RESIZE_CURSOR ) );
	  }
	  else
	  if ( S )
	  {
		setCursor( Cursor
			.getPredefinedCursor( Cursor.S_RESIZE_CURSOR ) );
	  }
	  else
	  {
		setCursor( Cursor
			.getPredefinedCursor( Cursor.MOVE_CURSOR ) );
	  }
	}
  }

  /**
   * Mouse has been dragged. Resize or move the component.
   *
   * @param e MouseEvent The mouse event.
   */
  private void this_mouseDragged( MouseEvent e )
  {
	Point loc = getLocation();
	Dimension dim = getSize();

	if ( W )
	{
	  int oldX = getLocation().x;
	  int newX = e.getX()+loc.x;
	  setBounds( newX, loc.y, dim.width+oldX-newX, dim.height );
	}
	if ( E )
	{
	  setSize( e.getPoint().x, dim.height );
	}

	loc = getLocation();
	dim = getSize();
	if ( N )
	{
	  int oldY = getLocation().y;
	  int newY = e.getY()+loc.y;
	  setBounds( loc.x, newY, dim.width, dim.height+oldY-newY );
	}
	if ( S )
	{
	  setSize( dim.width, e.getPoint().y );
	}

	if ( ! ( W|N|S|E ) )
	{
	  if ( lastDragPos==null )
	  {
		lastDragPos = e.getPoint();
	  }
	  else
	  {
		Point q = getLocation();
		q.translate( e.getPoint().x-lastDragPos.x, e.getPoint().y
			-lastDragPos.y );

		CTMComponentView view = getViewComponent();
		if ( view.isSnapToGrid( e ) )
		{
		  int gms = view.getGridMinorSpacing();
		  q.x = ( ( int ) ( q.x+gms/2 )/gms )*gms;
		  q.y = ( ( int ) ( q.y+gms/2 )/gms )*gms;
		}

		setLocation( q );
	  }
	}

	validate();

	getViewComponent().checkSize();
  }

  /**
   * Synchronize this component with its data.
   */
  public void sync()
  {
	int cp = textArea.getCaretPosition();
	int ss = textArea.getSelectionStart();
	int se = textArea.getSelectionEnd();

	CTMDataComment dc = ( CTMDataComment ) getData();
	textArea.setText( dc.getComment() );

	textArea.setCaretPosition( cp );
	textArea.setSelectionStart( ss );
	textArea.setSelectionEnd( se );

	setBounds( ( ( CTMDataComment ) getData() ).getBounds() );

	super.sync();
  }

  /**
   * Draw this component. Overriden to draw a border if the component is empty
   * or selected.
   *
   * @param g Graphics Graphics context to draw into.
   */
  public void paint( Graphics g )
  {
	super.paint( g );

	if ( isFocusOwner()||textArea.isFocusOwner()
		||textArea.getText().length()==0 )
	{
	  Graphics2D g2 = ( Graphics2D ) g;
	  g2.setColor( Color.black );
	  g2
		  .setStroke( new BasicStroke( 1.f, BasicStroke.CAP_BUTT,
		  BasicStroke.JOIN_BEVEL, 0.f,
		  new float[]
		  {
		  4.f, 3.f}, 0.f ) );
	  g2.drawRect( 0, 0, getWidth()-1, getHeight()-1 );
	}
  }
}


/**
 * Class to override handling of event bubbling for text field. This allows the
 * Comment box to be dragged via its text field.
 */
class MyTextArea
	extends JTextArea
{
  /**
   * Whether or not to bubble events to parent container.
   */
  private boolean bubbleEvents = false;

  /**
   * Check whether coordinates lie in the text areas bounds, but only if
   * bubble events are off.
   *
   * @param x
   *            int X coordinate to check.
   * @param y
   *            int Y coordinate to check.
   * @return boolean
   */
  public boolean contains( int x, int y )
  {
	if ( bubbleEvents )
	{
	  return super.contains( x, y );
	}
	return false;
  }

  /**
   * Set whether or not to bubble events to parent container.
   *
   * @param be boolean true = bubble events to parent container.
   */
  public void setBubbleEvents( boolean be )
  {
	this.bubbleEvents = be;
  }
}
