package turingmachine;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * Component used to show drag select areas.
 */
public class CTMComponentDragSelectArea
	extends JComponent
{
  /**
   * Mouse Pointer offset from the center of the state when it gets dragged.
   */
  private Point mDragOffset = new Point( -1, -1 );

  /**
   * Default constructor.
   */
  public CTMComponentDragSelectArea()
  {
	// tell swing that this component doesn't draw its complete area
	setOpaque( false );

	addMouseMotionListener( new MouseMotionAdapter()
	{
	  public void mouseDragged( MouseEvent e )
	  {
		this_mouseDragged( e );
	  }
	} );

	addMouseListener( new MouseAdapter()
	{
	  public void mousePressed( MouseEvent e )
	  {
		this_mousePressed( e );
	  }

	  public void mouseReleased( MouseEvent e )
	  {
		this_mouseReleased( e );
	  }
	} );

	setFocusable( false );
  }

  /**
   * Paint the view.
   *
   * @param g The graphics context to paint to.
   */
  public void paint( Graphics g )
  {
	Graphics2D g2 = ( Graphics2D ) g;
	g2.setColor( Color.black );
	g2.setStroke( new BasicStroke( 1.f, BasicStroke.CAP_BUTT,
		BasicStroke.JOIN_BEVEL, 0.f, new float[]
		{
		4.f, 3.f}, 0.f ) );
	g2.drawRect( 0, 0, getWidth()-1, getHeight()-1 );
  }

  /**
   * Define the precise shape of this component for the sake of mouse
   * processing.
   *
   * @param x The x coordinate to test.
   * @param y The y coordinate to test.
   * @return true if it does contain point(x,y) else false.
   */
  public boolean contains( int x, int y )
  {
	return ( ( x>=0 )&& ( x<getWidth() )&& ( y>=0 )&& ( y<getHeight() ) );
  }

  /**
   * Mouse pressed.
   *
   * @param e The mouse press event.
   */
  private void this_mousePressed( MouseEvent e )
  {
	if ( getViewComponent().getSelectedTool()==CTMFrame.TOOL_OPTION.HAND )
	{
	  // set the drag started position
	  mDragOffset.setLocation( e.getPoint() );

	  // bring component to front
	  Container j = getParent();
	  j.remove( this );
	  j.add( this, 0 );
	}
  }

  /**
   * Handle a the mouse being dragged.
   *
   * @param e The mouse drag event.
   */
  private void this_mouseDragged( MouseEvent e )
  {
	if ( ( e.getModifiers()&MouseEvent.BUTTON1_MASK )!=0 )
	{
	  if ( getViewComponent().getSelectedTool()==CTMFrame.TOOL_OPTION.HAND )
	  {
		// translate the point into view coordinates
		e.translatePoint( getX()-mDragOffset.x, getY()-mDragOffset.y );
		setLocation( e.getPoint() );
	  }
	}
  }

  /**
   * Handle mouse released on state.
   *
   * @param e The mouse released event.
   */
  private void this_mouseReleased( MouseEvent e )
  {}

  /**
   * Get the view component this link component belongs to.
   *
   * @return The view component that this link component belongs to.
   */
  public CTMComponentView getViewComponent()
  {
	return ( CTMComponentView ) getParent();
  }

}
