package turingmachine;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * Popup menu for right click on state.
 */
class CTMComponentPopupMenu
	extends JPopupMenu implements ActionListener
{
  /**
   * Menu Option instance.
   */
  private JMenuItem menuItem_Delete, menuItem_SetType_StartState,
	  menuItem_SetType_NormalState, menuItem_SetType_EndState,
	  menuItem_SetType_GosubState, menuItem_BreakPoint, menuItem_Rename,
	  menuItem_SendToBack, menuItem_BringToFront;

  /**
   * Constructor.
   *
   * @param aInvoker The component that invoked this popup.
   */
  public CTMComponentPopupMenu( Component aInvoker )
  {
	setInvoker( aInvoker );

	JMenu menu;

	if ( aInvoker instanceof CTMComponentState )
	{
	  menuItem_Rename = new JMenuItem( "Rename" );
	  menuItem_Rename.addActionListener( this );
	  add( menuItem_Rename );

	  menu = new JMenu( "Set Type" );
	  addSeparator();

	  menuItem_SetType_StartState = new JMenuItem( "Start State" );
	  menuItem_SetType_StartState.addActionListener( this );
	  menu.add( menuItem_SetType_StartState );

	  menuItem_SetType_NormalState = new JMenuItem( "Normal State" );
	  menuItem_SetType_NormalState.addActionListener( this );
	  menu.add( menuItem_SetType_NormalState );

	  menuItem_SetType_EndState = new JMenuItem( "End State" );
	  menuItem_SetType_EndState.addActionListener( this );
	  menu.add( menuItem_SetType_EndState );

	  menuItem_SetType_GosubState = new JMenuItem( "Gosub State" );
	  menuItem_SetType_GosubState.addActionListener( this );
	  menu.add( menuItem_SetType_GosubState );

	  add( menu );
	}

	menuItem_SendToBack = new JMenuItem( "Send To Back" );
	menuItem_SendToBack.addActionListener( this );
	add( menuItem_SendToBack );

	menuItem_BringToFront = new JMenuItem( "Bring To Front" );
	menuItem_BringToFront.addActionListener( this );
	add( menuItem_BringToFront );

	menuItem_Delete = new JMenuItem( "Delete" );
	menuItem_Delete.addActionListener( this );
	add( menuItem_Delete );

	addSeparator();

	menuItem_BreakPoint = new JMenuItem( "BreakPoint" );
	menuItem_BreakPoint.addActionListener( this );
	add( menuItem_BreakPoint );
  }

  /**
   * Handles any menu actions.
   *
   * @param aEvent The menu event.
   */
  public void actionPerformed( ActionEvent aEvent )
  {
	try
	{
	  CTMData data = ( ( CTMComponent ) getInvoker() ).getData();
	  Object source = aEvent.getSource();

	  if ( source==menuItem_Delete )
	  {
		data.delete();
	  }
	  else
	  if ( source==menuItem_SendToBack )
	  {
		// toggle breakpoint set
		data.getDocument().componentToBack( data );
	  }
	  else
	  if ( source==menuItem_BringToFront )
	  {
		// Todo: check state name not already taken.
		data.getDocument().componentToFront( data );
	  }

	  if ( data instanceof CTMDataState )
	  {
		CTMDataState sourceState = ( CTMDataState ) data;

		if ( source==menuItem_SetType_StartState )
		{
		  sourceState.setType( sourceState.START_NODE );
		}
		else
		if ( source==menuItem_SetType_NormalState )
		{
		  sourceState.setType( sourceState.NORMAL_NODE );
		}
		else
		if ( source==menuItem_SetType_EndState )
		{
		  sourceState.setType( sourceState.END_NODE );
		}
		else
		if ( source==menuItem_SetType_GosubState )
		{
		  sourceState.setType( sourceState.GOSUB_NODE );
		}
		else
		if ( source==menuItem_BreakPoint )
		{
		  // toggle breakpoint set
		  sourceState.setBreakPoint( !sourceState.isBreakPointSet() );
		}
		else
		if ( source==menuItem_Rename )
		{
		  // Todo: check state name not already taken.
		  String newName = JOptionPane.showInputDialog( this,
						   "Enter new state name:",
						   sourceState.getName() );
		  if ( newName!=null )
		  {
			sourceState.setStateName( newName );
		  }
		}
	  }
	}
	catch ( Exception ex )
	{
	  ex.printStackTrace();
	}
  }
}
