package turingmachine;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

import static turingmachine.CTMFrame.TOOL_OPTION;

/**
 * The state component.
 */
class CTMComponentState
	extends CTMComponent
{
  /**
   * Mouse Pointer offset from the center of the state when it gets dragged.
   */
  private Point mDragOffset = new Point();

  /**
   * The states state name text.
   */
  private JLabel mStateLabel;

  /**
   * Popup menu singleton instance.
   */
  private static JPopupMenu mPopup = null;

  private static Cursor cursor_makeLinkSrc = CTMUtility.createCursor(
	  "Resource/Cursor/makeLinkSrc.gif", new Point( 8, 8 ), "Make Link" ),
	  cursor_makeLinkDst = CTMUtility.createCursor(
						   "Resource/Cursor/makeLinkDst.gif", new Point( 8, 8 ),
						   "Make Link" );

  /**
   * Constructor.
   *
   * @param dataState The data for this component.
   */
  public CTMComponentState( CTMDataState dataState )
  {
	super( dataState );

	CTMPreferences prefs = getDocument().getPreferences();

	Font labelFont = new Font( "", Font.PLAIN, prefs.state_font_size );

	setSize( 48, 40 );

	setOpaque( false );

	mStateLabel = new JLabel( getDataState().getName(),
				  SwingConstants.CENTER );
	mStateLabel.setSize( getSize() );
	mStateLabel.setFont( labelFont );
	mStateLabel.setForeground( prefs.state_font_colour );
	mStateLabel.setOpaque( false );
	add( mStateLabel );

	addMouseMotionListener( new MouseMotionAdapter()
	{
	  public void mouseDragged( MouseEvent e )
	  {
		this_mouseDragged( e );
	  }
	} );

	addMouseListener( new MouseAdapter()
	{
	  public void mousePressed( MouseEvent e )
	  {
		// TODO: is this correct, I thought a mouse click should auto
		// focus
		requestFocus();

		this_mousePressed( e );
	  }

	  public void mouseReleased( MouseEvent e )
	  {
		this_mouseReleased( e );
	  }
	} );

	this.addFocusListener( new FocusListener()
	{
	  public void focusGained( FocusEvent e )
	  {
		repaint();
	  }

	  public void focusLost( FocusEvent e )
	  {
		repaint();
	  }
	} );

	this.setFocusable( true );
  }

  /**
   * Returns the best cursor for the current tool mode.
   *
   * @return Cursor for current tool context.
   */
  public Cursor getCursor()
  {
	CTMFrame.TOOL_OPTION selectedTool = getViewComponent().getSelectedTool();

	if ( selectedTool==CTMFrame.TOOL_OPTION.CREATE_LINK )
	{
	  return getViewComponent().getArrowComponent().isDragging()
		  ?cursor_makeLinkDst
		  :cursor_makeLinkSrc;
	}

	if ( selectedTool==CTMFrame.TOOL_OPTION.CREATE_LINK||
		selectedTool==CTMFrame.TOOL_OPTION.CREATE_COMMENT||
		selectedTool==CTMFrame.TOOL_OPTION.CREATE_NODE||
		selectedTool==CTMFrame.TOOL_OPTION.HAND )
	{
	  return Cursor.getPredefinedCursor( Cursor.MOVE_CURSOR );
	}

	return Cursor.getPredefinedCursor( Cursor.DEFAULT_CURSOR );
  }

  /**
   * Set this states state name.
   *
   * @param aStateName The state name to set.
   */
  public void setStateName( String aStateName )
  {
	mStateLabel.setText( aStateName );
  }

  /**
   * Paint the component.
   *
   * @param g The graphics context to paint to.
   */
  public void paint( Graphics g )
  {
	CTMDataDocument doc = getDocument();
	CTMDataState dataState = getDataState();
	CTMPreferences prefs = getDocument().getPreferences();
	int diameter = prefs.state_diameter;

	Graphics2D g2 = ( Graphics2D ) g;

	Dimension dZoom = getSize();
	int centerLeft = ( dZoom.width-diameter )/2;
	int centerTop = ( dZoom.height-diameter )/2;

	if ( dataState.getType()==CTMDataState.GOSUB_NODE )
	{
	  g2.setStroke( new BasicStroke( prefs.state_borderWidth ) );

	  // draw filled rect
	  g2.setColor( hasFocus()?Color.BLACK:prefs.state_colour );
	  g2.fillRect( centerLeft, centerTop, diameter, diameter );

	  // draw border
	  g2.setColor( hasFocus()?Color.WHITE:prefs.state_borderColour );
	  g2.drawRect( centerLeft, centerTop, diameter, diameter );
	}
	else
	{
	  g2.setStroke( new BasicStroke( prefs.state_borderWidth ) );

	  // draw filled circle
	  g2
		  .setColor( hasFocus()?Color.BLACK: ( dataState
		  .isBreakPointSet()?prefs.state_breakpointColour
		  :prefs.state_colour ) );

	  g2.fillOval( centerLeft, centerTop, diameter, diameter );

	  // draw border
	  g2.setColor( hasFocus()?Color.WHITE:prefs.state_borderColour );
	  g2.drawOval( centerLeft, centerTop, diameter, diameter );

	  // draw double border for end state
	  if ( dataState.getType()==CTMDataState.END_NODE )
	  {
		g2.drawOval( centerLeft+3, centerTop+3, diameter-6,
			diameter-6 );
	  }
	  else
	  if ( dataState.getType()==CTMDataState.START_NODE )
	  {
		// draw arrow on start state
		int c = centerTop+diameter/2;
		g2.drawLine( centerLeft, c, centerLeft-10, c );
		g2.drawLine( centerLeft, c, centerLeft-6, c-4 );
		g2.drawLine( centerLeft, c, centerLeft-6, c+4 );
	  }
	}

	// TODO: yuk
	mStateLabel.setForeground( hasFocus()?Color.WHITE:Color.BLACK );

	super.paint( g );
  }

  /**
   * Sync state component with its data.
   */
  public void sync()
  {
	CTMDataState dataState = getDataState();

	// update the states position to reflect aStates data position
	setLocation( translateFromCenter( dataState.getLocation() ) );

	// update the states title to reflect aStates title
	setStateName( dataState.getName() );

	super.sync();
  }

  /**
   * Mouse pressed.
   *
   * @param e The mouse press event.
   */
  private void this_mousePressed( MouseEvent e )
  {
	// if right click show context menu
	if ( ( e.getModifiers()&MouseEvent.BUTTON1_MASK )==0 )
	{
	  // else must be context click, invoke state popup
	  JPopupMenu popup = new CTMComponentPopupMenu( this );
	  popup.show( this, e.getX()+2, e.getY()+2 );
	}
	else
	{
	  switch ( getViewComponent().getSelectedTool() )
	  {
		case CREATE_NODE:
		case HAND:
		case CREATE_COMMENT:


		  // set the drag started position
		  mDragOffset.setLocation( e.getPoint() );


		  // bring the state to the front
		  getDataState().toFront();
		  break;

		  // if the create link toolbar option is selected
		case CREATE_LINK:


		  // set the drag started position
		  e.translatePoint( getLocation().x, getLocation().y );
		  CTMComponentArrow arrowComponent = getViewComponent()
			  .getArrowComponent();
		  arrowComponent.set( e.getPoint(), e.getPoint() );
		  arrowComponent.setDragging( true );
		  arrowComponent.setVisible( true );
		  mDragOffset.setLocation( e.getPoint() );
		  break;

		  // if the delete toolbar option is selected
		case DELETE:
		  CTMComponentState c = ( CTMComponentState ) e.getSource();
		  c.getDataState().delete();


		  //getDocument().notifyObservers();
		  break;
	  }
	}

	e.consume();
  }

  /**
   * Handle a the mouse being dragged.
   *
   * @param e The mouse drag event.
   */
  private void this_mouseDragged( MouseEvent e )
  {
	if ( ( e.getModifiers()&MouseEvent.BUTTON1_MASK )!=0 )
	{
	  CTMComponentView view = getViewComponent();
	  switch ( view.getSelectedTool() )
	  {
		case CREATE_NODE:
		case HAND:
		case CREATE_COMMENT:


		  // translate the point into view coordinates
		  e.translatePoint( getX()-mDragOffset.x, getY()
			  -mDragOffset.y );

		  Point pt = translateToCenter( e.getPoint() );
		  if ( view.isSnapToGrid( e ) )
		  {
			int gms = view.getGridMinorSpacing();
			pt.x = ( ( int ) ( pt.x+gms/2 )/gms )*gms;
			pt.y = ( ( int ) ( pt.y+gms/2 )/gms )*gms;
		  }

		  getDataState().setLocation( pt );
		  getDocument().notifyObservers();
		  break;

		case CREATE_LINK:
		  CTMComponentArrow arrowComponent = getViewComponent()
			  .getArrowComponent();
		  e.translatePoint( getLocation().x, getLocation().y );
		  arrowComponent.set( mDragOffset, e.getPoint() );
		  break;
	  }
	}
  }

  /**
   * Handle mouse released on state
   *
   * @param e The mouse released event.
   */
  private void this_mouseReleased( MouseEvent e )
  {
	// if left click
	if ( ( e.getModifiers()&MouseEvent.BUTTON1_MASK )!=0 )
	{
	  CTMComponentView view = getViewComponent();
	  switch ( view.getSelectedTool() )
	  {
		case CREATE_NODE:
		case HAND:
		case CREATE_COMMENT:
		  CTMComponentState source = ( CTMComponentState ) e.getSource();
		  getDataState().setLocation(
			  translateToCenter( source.getLocation() ) );

		  Point pt = translateToCenter( source.getLocation() );
		  if ( view.isSnapToGrid( e ) )
		  {
			int gms = view.getGridMinorSpacing();
			pt.x = ( ( int ) ( pt.x+gms/2 )/gms )*gms;
			pt.y = ( ( int ) ( pt.y+gms/2 )/gms )*gms;
		  }
		  getDataState().setLocation( pt );
		  break;

		case CREATE_LINK:
		  CTMComponentArrow arrowComponent = getViewComponent()
			  .getArrowComponent();
		  arrowComponent.setDragging( false );
		  arrowComponent.setVisible( false );

		  e.translatePoint( getLocation().x, getLocation().y );
		  Component c = getViewComponent().getComponentAt( e.getX(),
						e.getY() );

		  if ( c!=null&&c instanceof CTMComponentState )
		  {
			CTMDataLink dataLink = getDataState().linkAdd(
				( ( CTMComponentState ) c ).getDataState() );
			dataLink.addTransition();
		  }
		  break;
	  }
	}
  }

  /**
   * Translate a top-left corner point to a center point.
   *
   * @param aPoint the point to translate. aPoint is not modified.
   * @return A new instance of point translated to center.
   */
  private CTMPoint translateToCenter( Point aPoint )
  {
	CTMPoint p = new CTMPoint( aPoint );
	p.add( getWidth()/2, getHeight()/2 );

	return p;
  }

  /**
   * Translate a center point to a top-left corner point.
   *
   * @param aPoint The point to translate. aPoint is not modified.
   * @return A new instance of point translated to top-left corner.
   */
  private CTMPoint translateFromCenter( Point aPoint )
  {
	CTMPoint p = new CTMPoint( aPoint );
	p.subtract( getWidth()/2, getHeight()/2 );

	return p;
  }

  /**
   * Define the precise shape of this component for the sake of mouse
   * processing.
   *
   * @param x The x coordinate to test.
   * @param y The y coordinate to test.
   * @return true - it does containt point(x,y), else false.
   */
  public boolean contains( int x, int y )
  {
	int diameter = getDocument().getPreferences().state_diameter;
	Dimension dZoom = getSize();
	int tx = x-dZoom.width/2;
	int ty = y-dZoom.height/2;

	if ( 4* ( tx*tx+ty*ty )< ( diameter*diameter ) )
	{
	  return true;
	}

	return false;
  }

  /**
   * Get the state data this component represents.
   *
   * @return The state data this component represents.
   */
  public CTMDataState getDataState()
  {
	return ( CTMDataState ) getData();
  }
}
