package turingmachine;

import java.awt.*;
import java.io.*;

/**
 * Commment data for corresponding CRM
 */
public class CTMDataComment extends CTMData implements Serializable
{
	static final long serialVersionUID = 4156858461392611674L;

	/**
	 * Comment data.
	 */
	private String comment;

	/**
	 * Comment bounds.
	 */
	private Rectangle bounds;

	/**
	 * Constructor.
	 * 
	 * @param document
	 *            CTMDocument Document this data belongs to.
	 * @param comment
	 *            String Comment text.
	 * @param bounds
	 *            Rectangle Rectangular area comment covers.
	 */
	public CTMDataComment(CTMDataDocument document, String comment, Rectangle bounds)
	{
		super(document);
		this.comment = comment;
		this.bounds = bounds;
	}

	/**
	 * Get top left hand corner of comment rectangle.
	 * 
	 * @return Point
	 */
	public Point getLocation()
	{
		return bounds.getLocation();
	}

	/**
	 * Get comment rectangle dimensions.
	 * 
	 * @return Dimension
	 */
	public Dimension getSize()
	{
		return bounds.getSize();
	}

	/**
	 * Set comment rectangle bounds.
	 * 
	 * @param bounds
	 *            New comment bounds.
	 */
	public void setBounds(Rectangle bounds)
	{
		this.bounds = bounds;
		touch();
	}

	/**
	 * Get comment rectangle bounds.
	 * 
	 * @return bounds Comment bounds.
	 */
	public Rectangle getBounds()
	{
		return bounds;
	}

	/**
	 * Set the comment text.
	 * 
	 * @param comment
	 *            String The new comment text.
	 */
	public void setComment(String comment)
	{
		this.comment = comment;
		super.touch();
	}

	/**
	 * Get the comment text.
	 * 
	 * @return String The comment text.
	 */
	public String getComment()
	{
		return comment;
	}
}
