package turingmachine;

import java.io.*;

/**
 * Data for the transition on a link between states.
 */
class CTMDataTransition
	extends CTMData implements Serializable
{
  /**
   * Serial id, so future version will load old files
   */
  static final long serialVersionUID = -3401860529314374699L;

  /**
   * Symbol to match.
   */
  private String symbol = " ";

  /**
   * Direction to go after match, or the replace symbol.
   */
  private String direction = " ";

  /**
   * Constructor.
   *
   * @param document Document this data transition belongs to.
   * @param dataLink The link this data transition belongs to.
   */
  public CTMDataTransition( CTMDataDocument document, CTMDataLink dataLink )
  {
	super( document, dataLink );
  }

  /**
   * Set the match symbol.
   *
   * @param symbol The match symbol.
   * @return true if the symbol was valid else false.
   */
  public boolean setSymbol( String symbol )
  {
	if ( this.symbol!=symbol )
	{
	  // make sure the match is valid for this document
	  if ( getDocument().getSymbols().indexOf( symbol )==-1 )
	  {
		return false;
	  }
	  this.symbol = symbol;
	  touch();
	}

	return true;
  }

  /**
   * Get the match symbol.
   *
   * @return The match symbol.
   */
  public String getMatch()
  {
	return symbol;
  }

  /**
   * Get the direction, or replace symbol.
   *
   * @param direction The direction or replace symbol to set.
   * @return true if the direction/replace symbol was valid else false.
   */
  public boolean setDirection( String direction )
  {
	if ( this.direction!=direction )
	{
	  // make sure the direction is valid for this document
	  if ( getDocument().getDirections().indexOf( direction )==-1
		  &&getDocument().getSymbols().indexOf( direction )==-1 )
	  {
		return false;
	  }
	  this.direction = direction;
	  touch();
	}

	return true;
  }

  /**
   * Delete this transition from its parent link.
   */
  public void delete()
  {
	super.delete();
	( ( CTMDataLink ) getDataParent() ).transitionDeleted( this );
  }

  /**
   * Get the direction or replace symbol.
   *
   * @return The direction or replace symbol.
   */
  public String getDirection()
  {
	return direction;
  }
}
