package turingmachine;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * Document properties modification dialog.
 */
class CTMDialogDocumentProperties
	extends JDialog implements ActionListener
{
  /**
   * Title label.
   */
  private JLabel jLabelTitle = new JLabel( "Title" );

  /**
   * Title text field.
   */
  private JTextField mTextFieldTitle = new JTextField();

  /**
   * Symbols label.
   */
  private JLabel jLabelSymbols = new JLabel( "Symbols" );

  /**
   * Symbols text field.
   */
  private JTextField mTextFieldSymbols = new JTextField();

  /**
   * Directions label.
   */
  private JLabel jLabelDirections = new JLabel( "Directions" );

  /**
   * Directions combo-box.
   */
  private JComboBox mComboBoxDirections = new JComboBox();

  /**
   * Whether or not the machine is deterministic checkbox.
   */
  private JCheckBox mCheckBoxDeterministic = new JCheckBox( "Deterministic" );

  /**
   * OK Button.
   */
  private JButton mButtonOK = new JButton( "OK" );

  /**
   * Cancel button.
   */
  private JButton mButtonCancel = new JButton( "Cancel" );

  /**
   * Whether or not the dialog had the OK button clicked.
   */
  private boolean wasOK = false;

  /**
   * The document that is having its properties set.
   */
  private CTMDataDocument mDocument;

  /**
   * Default constructor for JBuilder designer.
   */
  public CTMDialogDocumentProperties()
  {
	try
	{
	  jbInit();
	}
	catch ( Exception ex )
	{
	  ex.printStackTrace();
	}
  }

  /**
   * Constructor.
   *
   * @param aParent The parent frame for this dialog.
   * @param aDocument The document that is having its properties set.
   */
  public CTMDialogDocumentProperties( JFrame aParent, CTMDataDocument aDocument )
  {
	super( aParent );
	mDocument = aDocument;

	try
	{
	  jbInit();
	  pack();
	}
	catch ( Exception ex )
	{
	  ex.printStackTrace();
	}
  }

  private void jbInit()
	  throws Exception
  {
	setResizable( false );
	setTitle( "Turing Machine Properties" );
	getContentPane().setLayout( new GridBagLayout() );

	mTextFieldTitle
		.setText( ( mDocument==null )?"":mDocument.getTitle() );
	mTextFieldSymbols.setText( ( mDocument==null )?"":mDocument
		.getSymbols() );
	mCheckBoxDeterministic
		.setSelected( ( mDocument==null )?true:mDocument
		.isDeterministic() );

	mComboBoxDirections.addItem( "LR" );
	mButtonOK.addActionListener( this );
	mButtonCancel.addActionListener( this );
	jLabelDirections.setFont( new java.awt.Font( "Dialog", Font.BOLD, 11 ) );
	jLabelSymbols.setFont( new java.awt.Font( "Dialog", Font.BOLD, 11 ) );
	jLabelTitle.setFont( new java.awt.Font( "Dialog", Font.BOLD, 11 ) );
	this.getContentPane().add(
		jLabelDirections,
		new GridBagConstraints( 0, 3, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.NONE,
		new Insets( 4, 4, 4, 4 ), 0, 0 ) );
	this.getContentPane().add(
		jLabelTitle,
		new GridBagConstraints( 0, 1, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.NONE,
		new Insets( 4, 4, 4, 4 ), 0, 0 ) );
	this.getContentPane().add(
		jLabelSymbols,
		new GridBagConstraints( 0, 2, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.NONE,
		new Insets( 4, 4, 4, 4 ), 19, 7 ) );
	this.getContentPane().add(
		mComboBoxDirections,
		new GridBagConstraints( 1, 3, 2, 1, 1.0, 0.0,
		GridBagConstraints.CENTER,
		GridBagConstraints.HORIZONTAL, new Insets( 4, 4, 4, 4 ),
		0, 0 ) );
	this.getContentPane().add(
		mTextFieldSymbols,
		new GridBagConstraints( 1, 2, 2, 1, 1.0, 0.0,
		GridBagConstraints.CENTER,
		GridBagConstraints.HORIZONTAL, new Insets( 4, 4, 4, 4 ),
		0, 0 ) );
	this.getContentPane().add(
		mTextFieldTitle,
		new GridBagConstraints( 1, 1, 2, 1, 1.0, 0.0,
		GridBagConstraints.CENTER,
		GridBagConstraints.HORIZONTAL, new Insets( 4, 4, 4, 4 ),
		0, 0 ) );
	this.getContentPane().add(
		mButtonCancel,
		new GridBagConstraints( 1, 5, 1, 1, 1.0, 0.0,
		GridBagConstraints.EAST, GridBagConstraints.NONE,
		new Insets( 4, 4, 4, 4 ), 0, 0 ) );
	this.getContentPane().add(
		mButtonOK,
		new GridBagConstraints( 2, 5, 1, 1, 0.0, 0.0,
		GridBagConstraints.EAST, GridBagConstraints.NONE,
		new Insets( 4, 4, 4, 4 ), 0, 0 ) );
	this.getContentPane().add(
		mCheckBoxDeterministic,
		new GridBagConstraints( 1, 4, 1, 1, 0.0, 0.0,
		GridBagConstraints.WEST, GridBagConstraints.NONE,
		new Insets( 4, 4, 4, 4 ), 0, 0 ) );
  }

  // Close the dialog on a button event
  public void actionPerformed( ActionEvent e )
  {
	if ( e.getSource()==mButtonCancel )
	{
	  dispose();
	}
	else
	if ( e.getSource()==mButtonOK )
	{
	  ok();
	}
  }

  /**
   * Return whether or not the OK button was clicked.
   *
   * @return boolean True if OK button was clicked, else false.
   */
  public boolean wasOK()
  {
	return wasOK;
  }

  /**
   * The ok button action.
   */
  private void ok()
  {
	// TODO: Check all the values
	String symbols = mTextFieldSymbols.getText();
	for ( int i = 0; i<symbols.length(); i++ )
	{
	  if ( symbols.indexOf( symbols.charAt( i ), i+1 )!=-1 )
	  {
		JOptionPane.showMessageDialog( this,
			"Duplicate symbols.  All symbols must be unique.",
			"Invalid symbols", JOptionPane.ERROR_MESSAGE );
		return;
	  }
	}

	// set all the values
	mDocument.setTitle( mTextFieldTitle.getText() );
	mDocument.setSymbols( mTextFieldSymbols.getText() );
	mDocument.setDirections( ( String ) mComboBoxDirections.getSelectedItem() );
	mDocument.setIsDeterministic( mCheckBoxDeterministic.isSelected() );

	wasOK = true;
	dispose();
  }

}
