package turingmachine;

import java.io.*;

import javax.swing.filechooser.FileFilter;

/**
 * Simple file filter.
 */
class CTMFileFilter
	extends FileFilter
{
  /**
   * List of extensions supported by this filter.
   */
  private String[] mExtensions;

  /**
   * Human readable description.
   */
  private String mDescription;

  /**
   * Constructor.
   *
   * @param aDescription Human readable description of filter.
   * @param aExtensions Entensions allowed by this filter.
   */
  public CTMFileFilter( String aDescription, String[] aExtensions )
  {
	mDescription = aDescription;
	mExtensions = aExtensions;
  }

  /**
   * Check whether or not this file type is an accepted type.
   *
   * @param aFile The file that is to be tested.
   * @return true, the file is an accepted type else false.
   */
  public boolean accept( File aFile )
  {
	// Directories are always accepted so we can browse
	if ( aFile.isDirectory() )
	{
	  return true;
	}

	// Get the file extension
	String filename = aFile.getName();
	int extPos = filename.lastIndexOf( "." );
	if ( extPos==-1 )
	{
	  return false;
	}
	String ext = filename.substring( extPos+1 );

	// Check the extension
	for ( int i = 0; i<mExtensions.length; i++ )
	{
	  if ( ext.compareToIgnoreCase( mExtensions[i] )==0 )
	  {
		return true;
	  }
	}

	return false;
  }

  /**
   * Get the human readable form of what extensions this filter supports.
   *
   * @return The description of this extension type.
   */
  public String getDescription()
  {
	return mDescription;
  }
}
