package turingmachine;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;

/**
 * Main application frame for turing machine.
 */
class CTMFrame
	extends JFrame implements ActionListener
{
  /**
   * Desktop pane to hold turing machine views.
   */
  public JDesktopPane mDesktopPane;

  /**
   * Tool button.
   */
  private JToggleButton mHandButton, mStateButton, mLinkButton,
	  mDeleteButton, mCommentButton;

  /**
   * Tool Id
   */
  public static enum TOOL_OPTION
  {
	HAND, CREATE_NODE, CREATE_LINK, DELETE, CREATE_COMMENT
  }


  /**
   * Currently selected tool option.
   */
  private TOOL_OPTION mToolOption;

  /**
   * Constructor.
   */
  public CTMFrame()
  {
	// setIconImage(Toolkit.getDefaultToolkit().createImage(CTMFrame.class.getResource("[Your
	// Icon]")));

	setSize( new Dimension( 640, 480 ) );
	setTitle( "Turing Machine Simulator" );

	setJMenuBar( new CTMMenuBar() );

	JToolBar mToolbar = new JToolBar();
	mToolbar.setFloatable( true );

	ButtonGroup group = new ButtonGroup();

	ImageIcon toolHand = new ImageIcon( turingmachine.CTMFrame.class
						 .getResource( "Resource/Tool/hand.gif" ) );
	mHandButton = new JToggleButton( toolHand );
	mHandButton.setToolTipText( "Move Component" );
	mHandButton.addActionListener( this );
	mHandButton.setBackground( Color.white );
	mToolbar.add( mHandButton );
	group.add( mHandButton );

	ImageIcon toolState = new ImageIcon( turingmachine.CTMFrame.class
						  .getResource( "Resource/Tool/state.gif" ) );
	mStateButton = new JToggleButton( toolState );
	mStateButton.setToolTipText( "Create State" );
	mStateButton.addActionListener( this );
	mStateButton.setBackground( Color.white );
	mToolbar.add( mStateButton );
	group.add( mStateButton );

	ImageIcon toolLink = new ImageIcon( turingmachine.CTMFrame.class
						 .getResource( "Resource/Tool/link.gif" ) );
	mLinkButton = new JToggleButton( toolLink );
	mLinkButton.setToolTipText( "Create Link" );
	mLinkButton.addActionListener( this );
	mLinkButton.setBackground( Color.white );
	mToolbar.add( mLinkButton );
	group.add( mLinkButton );

	ImageIcon toolComment = new ImageIcon( turingmachine.CTMFrame.class
							.getResource( "Resource/Tool/comment.gif" ) );
	mCommentButton = new JToggleButton( toolComment );
	mCommentButton.setToolTipText( "Create comment " );
	mCommentButton.addActionListener( this );
	mCommentButton.setBackground( Color.white );
	mToolbar.add( mCommentButton );
	group.add( mCommentButton );

	ImageIcon toolDelete = new ImageIcon( turingmachine.CTMFrame.class
						   .getResource( "Resource/Tool/delete.gif" ) );
	mDeleteButton = new JToggleButton( toolDelete );
	mDeleteButton.setToolTipText( "Delete State, Link or Comment" );
	mDeleteButton.addActionListener( this );
	mDeleteButton.setBackground( Color.white );
	mToolbar.add( mDeleteButton );
	group.add( mDeleteButton );

	group.setSelected( mStateButton.getModel(), true );
	mToolOption = TOOL_OPTION.CREATE_NODE;

	getContentPane().add( mToolbar, BorderLayout.NORTH );

	mDesktopPane = new JDesktopPane();
	getContentPane().add( mDesktopPane );

	// mDesktopPane.add( new CTMInternalFrameTape() );

	setDefaultCloseOperation( JFrame.DO_NOTHING_ON_CLOSE );

	addWindowListener( new WindowAdapter()
	{
	  public void windowClosing( WindowEvent e )
	  {
		if ( !closeAllInternalFrames() )
		{
		  // TODO there has to be a better way
		  throw new RuntimeException( "Quit cancelled" );
		}

		System.exit( 0 );
	  }
	} );
  }


  /**
   * Close all internal frames open in this Frame.
   *
   * @return true if all internal frames closed else false.
   */
  public boolean closeAllInternalFrames()
  {
	JInternalFrame frames[] = getDesktopPane().getAllFrames();

	// close all documents belonging to this app.
	for ( int i = 0; i<frames.length; i++ )
	{
	  frames[i].doDefaultCloseAction();
	}

	return getDesktopPane().getAllFrames().length==0;
  }

  /**
   * Checks whether a file is already open by the application.
   *
   * @param filename String Filename of file.
   * @return boolean true if it is open.
   */
  public boolean fileIsAlreadyOpen( String filename )
  {
	JInternalFrame frames[] = getDesktopPane().getAllFrames();

	// close all documents belonging to this app.
	for ( int i = 0; i<frames.length; i++ )
	{
	  CTMInternalFrameView cif = ( CTMInternalFrameView ) frames[i];
	  if ( cif.getDocument().getFilename().compareTo( filename )==0 )
	  {
		return true;
	  }
	}

	return false;
  }

  /**
   * handle tool bar selections.
   *
   * @param aEvent The information about which tool caused the event.
   */
  public void actionPerformed( ActionEvent aEvent )
  {
	if ( aEvent.getSource()==mStateButton )
	{
	  mToolOption = TOOL_OPTION.CREATE_NODE;
	}
	else
	if ( aEvent.getSource()==mLinkButton )
	{
	  mToolOption = TOOL_OPTION.CREATE_LINK;
	}
	else
	if ( aEvent.getSource()==mDeleteButton )
	{
	  mToolOption = TOOL_OPTION.DELETE;
	}
	else
	if ( aEvent.getSource()==mHandButton )
	{
	  mToolOption = TOOL_OPTION.HAND;
	}
	else
	if ( aEvent.getSource()==mCommentButton )
	{
	  mToolOption = TOOL_OPTION.CREATE_COMMENT;
	}
  }

  /**
   * Get the desktop pane for the frame.
   *
   * @return This frames desktop pane.
   */
  public JDesktopPane getDesktopPane()
  {
	return mDesktopPane;
  }

  /**
   * Gets the currently selected tool from the toolbar.
   *
   * @return The tool currently in use.
   */
  public TOOL_OPTION getSelectedTool()
  {
	return mToolOption;
  }
}
