package turingmachine;

import java.awt.*;
import java.util.*;

import javax.swing.*;
import javax.swing.event.*;

/**
 * A frame that holds a view component.
 */
class CTMInternalFrameView
	extends JInternalFrame
{
  /**
   * The scrollable area for the frame.
   */
  private JScrollPane mScrollPane;

  /**
   * The view component in this frame.
   */
  private CTMComponentView mViewComponent;

  /**
   * The document this frame belongs to.
   */
  private CTMDataDocument mDocument;

  /**
   * Constructor.
   *
   * @param aDocument The document that this frame contains a view of.
   */
  public CTMInternalFrameView( CTMDataDocument aDocument )
  {
	super( "", true, true, true, true );

	mDocument = aDocument;
	setTitle( mDocument.getTitle() );

	// create view component
	mViewComponent = new CTMComponentView( getDocument() );

	// create scroll component containing view component
	mScrollPane = new JScrollPane( mViewComponent,
				  JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
				  JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );

	// add scroll pane to frame.
	getContentPane().add( mScrollPane );

	// update title to reflect the documents title
	updateTitle();

	getDocument().addObserver( new Observer()
	{
	  public void update( Observable o, Object arg )
	  {
		sync();
	  }
	} );

	this.addInternalFrameListener( new InternalFrameAdapter()
	{
	  public void internalFrameClosing( InternalFrameEvent e )
	  {
		int count = 0;
		JInternalFrame frames[] = getDesktopPane().getAllFrames();

		// close all documents belonging to this app.
		for ( int i = 0; i<frames.length; i++ )
		{
		  if ( frames[i] instanceof CTMInternalFrameView )
		  {
			CTMInternalFrameView view = ( CTMInternalFrameView ) frames[i];
			if ( view.getDocument()==getDocument() )
			{
			  count++;
			}
		  }
		}

		if ( count==1 )
		{
		  if ( getDocument().isModified()&&!getDocument().saveQuestion( getRootFrame() ) )
		  {
			throw new RuntimeException( "View close cancelled" );
		  }
		}
	  }
	} );
  }

  /**
   * Synchronise all sub-components of this frame.
   */
  public void sync()
  {
	updateTitle();
	getViewComponent().sync();
  }

  /**
   * Update frame title depending on document properties.
   */
  private void updateTitle()
  {
	CTMDataDocument doc = getDocument();
	String newTitle = doc.getTitle();

	if ( doc.isModified() )
	{
	  newTitle = newTitle+" *";
	}

	if ( getTitle().compareTo( newTitle )!=0 )
	{
	  setTitle( newTitle );
	}
  }

  /**
   * Get the view component this frame contains.
   *
   * @return This frames view component.
   */
  public CTMComponentView getViewComponent()
  {
	return mViewComponent;
  }

  /**
   * Get the document this frame is rendering.
   *
   * @return This frames document.
   */
  public CTMDataDocument getDocument()
  {
	return mDocument;
  }

  /**
   * Get the root frame this menubar belongs to.
   *
   * @return The root frame this menubar belongs to.
   */
  private CTMFrame getRootFrame()
  {
	Component c = this;
	while ( ! ( ( c = c.getParent() )instanceof CTMFrame ) )
	{
	  ;
	}
	return ( CTMFrame ) c;
  }
}
