package turingmachine;

import java.awt.*;
import java.awt.event.*;
import java.beans.*;

import javax.swing.*;
import javax.swing.event.*;

/**
 * Application main menubar.
 */
class CTMMenuBar
	extends JMenuBar implements ActionListener
{
  /**
   * Menu instance.
   */
  private JMenu menuFile, menuWindow, menuTuringMachine;

  /**
   * Menu Option instance.
   */
  private JMenuItem menuFile_New_TuringMachine, menuFile_Close,
	  menuFile_CloseAll, menuFile_Load, menuFile_Save, menuFile_SaveAs,
	  menuFile_Exit, menuHelp_About,
	  menuWindow_NewView, menuWindow_GridSpacing,
	  menuTuringMachine_Properties, menuTuringMachine_Colours,
	  menuTuringMachine_Validate;

  private JCheckBoxMenuItem menuWindow_Grid, menuWindow_SnapToGrid;

  /**
   * Constructor. Make the menubar and menus.
   */
  public CTMMenuBar()
  {
	// create and add the menu bar
	menuFile = new JMenu( "File" );
	JMenu menuFile_New = new JMenu( "New" );
	menuFile_New_TuringMachine = createMenuItem( menuFile_New,
		"Turing machine", true );

	menuFile.add( menuFile_New );
	menuFile_Load = createMenuItem( menuFile, "Load...", true );
	menuFile.addSeparator();
	menuFile_Close = createMenuItem( menuFile, "Close", true );
	menuFile_CloseAll = createMenuItem( menuFile, "Close All", true );
	menuFile.addSeparator();
	menuFile_Save = createMenuItem( menuFile, "Save", true );
	menuFile_SaveAs = createMenuItem( menuFile, "Save As...", true );
	menuFile.addSeparator();
	menuFile_Exit = createMenuItem( menuFile, "Exit", true );
	menuFile.addMenuListener( new MenuListener()
	{
	  public void menuSelected( MenuEvent e )
	  {
		// update the save/saveas menuItems
		// get the front most window
		CTMInternalFrameView selectedFrame = getSelectedFrame();

		if ( selectedFrame!=null )
		{
		  menuFile_Close.setEnabled( true );
		  menuFile_CloseAll.setEnabled( true );
		  menuFile_Save.setEnabled( selectedFrame.getDocument()
			  .isModified() );
		  menuFile_SaveAs.setEnabled( true );
		}
		else
		{
		  menuFile_Close.setEnabled( false );
		  menuFile_CloseAll.setEnabled( false );
		  menuFile_Save.setEnabled( false );
		  menuFile_SaveAs.setEnabled( false );
		}
	  }

	  public void menuCanceled( MenuEvent e )
	  {}

	  public void menuDeselected( MenuEvent e )
	  {}
	} );

	menuTuringMachine = new JMenu( "Turing Machine" );
	menuTuringMachine_Properties = createMenuItem( menuTuringMachine,
		"Properties...", true );
	menuTuringMachine_Colours = createMenuItem( menuTuringMachine,
		"Colours...", true );
	menuTuringMachine_Validate = createMenuItem( menuTuringMachine,
		"Validate", true );
	menuTuringMachine.addMenuListener( new MenuListener()
	{
	  public void menuSelected( MenuEvent e )
	  {
		menuTuringMachine_Properties
			.setEnabled( getSelectedFrame()!=null );
		menuTuringMachine_Validate
			.setEnabled( getSelectedFrame()!=null );
	  }

	  public void menuCanceled( MenuEvent e )
	  {}

	  public void menuDeselected( MenuEvent e )
	  {}
	} );

	menuWindow = new JMenu( "Window" );
	menuWindow_NewView = createMenuItem( menuWindow, "New View", true );
	menuWindow.addSeparator();
	menuWindow_Grid = createCheckboxMenuItem( menuWindow, "Grid", true );
	menuWindow_SnapToGrid = createCheckboxMenuItem( menuWindow,
							"Snap To Grid", true );
	menuWindow_GridSpacing = createMenuItem( menuWindow, "Grid Spacing",
							 true );

	menuWindow.addMenuListener( new MenuListener()
	{
	  public void menuSelected( MenuEvent e )
	  {
		CTMInternalFrameView selectedFrame = getSelectedFrame();

		boolean isAFrameSelected = selectedFrame!=null;

		menuWindow_NewView.setEnabled( isAFrameSelected );
		menuWindow_Grid.setEnabled( isAFrameSelected );
		menuWindow_SnapToGrid.setEnabled( isAFrameSelected );
		menuWindow_GridSpacing.setEnabled( isAFrameSelected );

		if ( isAFrameSelected )
		{
		  CTMComponentView view = selectedFrame.getViewComponent();
		  menuWindow_Grid.setState( view.isShowingGrid() );
		  menuWindow_SnapToGrid.setState( view.isSnapToGrid() );
		}
	  }

	  public void menuCanceled( MenuEvent e )
	  {}

	  public void menuDeselected( MenuEvent e )
	  {}
	} );

	JMenu menuHelp = new JMenu( "Help" );
	menuHelp_About = createMenuItem( menuHelp, "About", true );

	add( menuFile );
	add( menuTuringMachine );
	add( menuWindow );
	add( menuHelp );
  }

  /**
   * Creates a menuitem in aMenu.
   *
   * @param aMenu The menu to add the item to.
   * @param aName The text for the menu item.
   * @param aEnabled Whether or not this menu item is enabled.
   * @return The new menu item.
   */
  private JMenuItem createMenuItem( JMenu aMenu, String aName, boolean aEnabled )
  {
	JMenuItem mi = new JMenuItem( aName );
	mi.addActionListener( this );
	mi.setEnabled( aEnabled );
	aMenu.add( mi );

	return mi;
  }

  /**
   * Creates a checkbox menu item.
   *
   * @param aMenu The menu to add the item to.
   * @param aName The text for the menu item.
   * @param aEnabled Whether or not this menu item is enabled.
   * @return The new menu item.
   */
  private JCheckBoxMenuItem createCheckboxMenuItem( JMenu aMenu, String aName,
	  boolean aEnabled )
  {
	JCheckBoxMenuItem mi = new JCheckBoxMenuItem( aName );
	mi.addActionListener( this );
	mi.setEnabled( aEnabled );
	aMenu.add( mi );

	return mi;
  }

  /**
   * Get the currently selected frame.
   *
   * @return The frontmost window in the application.
   */
  private CTMInternalFrameView getSelectedFrame()
  {
	return ( CTMInternalFrameView ) ( getRootFrame().getDesktopPane().getSelectedFrame() );
  }

  /**
   * Handle menu actions.
   *
   * @param e The event holding the action information.
   */
  public void actionPerformed( ActionEvent e )
  {
	Object source = e.getSource();
	CTMInternalFrameView selectedFrame = getSelectedFrame();

	if ( source==menuFile_Exit )
	{
	  menuFile_Exit();
	}
	else
	if ( source==menuHelp_About )
	{
	  menuHelp_About();
	}
	else
	if ( source==menuFile_New_TuringMachine )
	{
	  menuFile_New_TuringMachine( null );
	}
	else
	if ( source==menuFile_Close )
	{
	  menuFile_Close();
	}
	else
	if ( source==menuFile_CloseAll )
	{
	  menuFile_CloseAll();
	}
	else
	if ( source==menuWindow_NewView )
	{
	  menuFile_New_TuringMachine( selectedFrame.getDocument() );
	}
	else
	if ( source==menuWindow_Grid )
	{
	  menuWindow_Grid();
	}
	else
	if ( source==menuWindow_SnapToGrid )
	{
	  menuWindow_SnapToGrid();
	}
	else
	if ( source==menuWindow_GridSpacing )
	{
	  menuWindow_GridSpacing();
	}
	else
	if ( source==menuTuringMachine_Properties )
	{
	  menuTuringMachine_Properties();
	}
	else
	if ( source==menuTuringMachine_Colours )
	{
	  menuTuringMachine_Colours();
	}
	else
	if ( source==menuTuringMachine_Validate )
	{
	  menuTuringMachine_Validate();
	}
	else
	if ( source==menuFile_Load )
	{
	  menuFile_Load();
	}
	else
	if ( source==menuFile_Save )
	{
	  selectedFrame.getDocument().save( getRootFrame() );
	}
	else
	if ( source==menuFile_SaveAs )
	{
	  selectedFrame.getDocument().saveAs( getRootFrame() );
	}
  }

  /**
   * File | Exit action performed.
   */
  private void menuFile_Exit()
  {
	menuFile_CloseAll();

	// only exit if all windows closed.
	if ( getSelectedFrame()==null )
	{
	  System.exit( 0 );
	}
  }

  /**
   * Creates new view on document.
   *
   * @param aDocument The document to create a new view of.<br>
   *            If this is null a new document is created.
   */
  private void menuFile_New_TuringMachine( CTMDataDocument aDocument )
  {
	CTMFrame parentFrame = getRootFrame();

	// if we weren't passed a document create a new one
	CTMDataDocument document = ( aDocument==null )?new CTMDataDocument():aDocument;

	// create the documents view
	CTMInternalFrameView internalFrame = new CTMInternalFrameView( document );

	// tile new internal frames
	Rectangle bounds;
	JInternalFrame selected = getRootFrame().getDesktopPane().getSelectedFrame();
	if ( selected!=null )
	{
	  bounds = selected.getBounds();
	  bounds.translate( 20, 20 );
	}
	else
	{
	  bounds = new Rectangle( 20, 20, 400, 300 );
	}

	internalFrame.setBounds( bounds );

	// add the internalFrame to the application window
	parentFrame.getDesktopPane().add( internalFrame );
	internalFrame.setVisible( true );

	try
	{
	  internalFrame.setSelected( true );
	}
	catch ( PropertyVetoException exception )
	{
	  System.err.println( exception );
	}

	// show the properties dialog if this is a new document
	if ( aDocument==null&&!menuTuringMachine_Properties() )
	{
	  // properties dialog was cancelled so close new document
	  internalFrame.dispose();
	  return;
	}

	// snyc the view with the document
	internalFrame.sync();
  }

  /**
   * Show the modify properties dialog for a document.
   *
   * @returntTrue if the dialog was actioned with the OK button.
   */
  private boolean menuTuringMachine_Properties()
  {
	CTMDialogDocumentProperties dlg = new CTMDialogDocumentProperties(
		getRootFrame(), getSelectedFrame().getDocument() );
	CTMUtility.centerContainer( dlg );
	dlg.setModal( true );
	dlg.setVisible( true );

	return dlg.wasOK();
  }

  /**
   * Show the modify colours dialog for a document.
   */
  private void menuTuringMachine_Colours()
  {
	JPanel optionsPanel = new JPanel( new BorderLayout() );
	JComboBox options = new JComboBox( new String[]
						{"Border", "View"} );
	optionsPanel.add( options, BorderLayout.NORTH );

	JColorChooser colourChooser = new JColorChooser();
	JDialog dlg = new JDialog( getRootFrame() );
	dlg.setSize( 320, 200 );
	CTMUtility.centerContainer( dlg );
	dlg.getContentPane().setLayout( new BorderLayout() );
	dlg.getContentPane().add( optionsPanel, BorderLayout.WEST );
	dlg.getContentPane().add( colourChooser, BorderLayout.EAST );
	dlg.pack();
	dlg.setVisible( false );
  }

  /**
   * Validate document menu item.
   */
  private void menuTuringMachine_Validate()
  {
	CTMInternalFrameView selectedFrame = getSelectedFrame();
	CTMDataDocument document = selectedFrame.getDocument();
	document.validate( getRootFrame() );
  }

  /**
   * Toggle grid status.
   */
  private void menuWindow_Grid()
  {
	CTMInternalFrameView selectedFrame = getSelectedFrame();
	if ( selectedFrame!=null )
	{
	  CTMComponentView view = selectedFrame.getViewComponent();
	  view.toggleGrid();
	  menuWindow_Grid.setState( view.isShowingGrid() );
	}
  }

  /**
   * Toggle snap-to-grid status.
   */
  private void menuWindow_SnapToGrid()
  {
	CTMInternalFrameView selectedFrame = getSelectedFrame();
	if ( selectedFrame!=null )
	{
	  CTMComponentView view = selectedFrame.getViewComponent();
	  view.toggleSnapToGrid();
	  menuWindow_SnapToGrid.setState( view.isSnapToGrid() );
	}
  }

  /**
   * Modify grid spacing.
   */
  private void menuWindow_GridSpacing()
  {
	CTMInternalFrameView selectedFrame = getSelectedFrame();
	if ( selectedFrame!=null )
	{
	  CTMComponentView view = selectedFrame.getViewComponent();
	  String result = ( String ) JOptionPane.showInputDialog( view,
					  "Enter a new grid spacing value", ""
					  +view.getGridMinorSpacing() );

	  try
	  {
		int newSpacing = Integer.parseInt( result );
		if ( newSpacing<4 )
		{
		  JOptionPane.showMessageDialog( view,
			  "Invalid number entered for grid spacing.  Must be greater than or equal to 4.",
			  "Error", JOptionPane.ERROR_MESSAGE );
		}
		else
		{
		  view.setGridMinorSpacing( newSpacing );
		}
	  }
	  catch ( NumberFormatException ex )
	  {
		JOptionPane.showMessageDialog( view,
			"Invalid number entered for grid spacing.", "Error",
			JOptionPane.ERROR_MESSAGE );
	  }
	}
  }

  /**
   * Display a load dialog.
   */
  private void menuFile_Load()
  {
	// create new document
	CTMDataDocument doc = CTMDataDocument.load( getRootFrame() );

	// if document create succeeded
	if ( doc!=null )
	{
	  menuFile_New_TuringMachine( doc );
	}
  }

  /**
   * Close the currently selected frame.
   */
  private void menuFile_Close()
  {
	getSelectedFrame().doDefaultCloseAction();
  }

  /**
   * Close all the currently open windows
   */
  public void menuFile_CloseAll()
  {
	getRootFrame().closeAllInternalFrames();
  }

  /**
   * Displays About dialog box.
   */
  private void menuHelp_About()
  {
	CTMDialogAbout dlg = new CTMDialogAbout( getRootFrame() );
	CTMUtility.centerContainer( dlg );
	dlg.setVisible( true );
  }

  /**
   * Get the root frame this menubar belongs to.
   *
   * @return The root frame this menubar belongs to.
   */
  private CTMFrame getRootFrame()
  {
	Component c = this;
	while ( ! ( ( c = c.getParent() )instanceof CTMFrame ) )
	{
	  ;
	}
	return ( CTMFrame ) c;
  }
}
