package turingmachine;

import java.awt.*;
import java.awt.geom.*;
import java.io.*;

/**
 * New point class to implement extra stuff default point class implements.
 */
class CTMPoint
	extends Point implements Serializable
{
  /**
   * Serial id, so future version will load old files.
   */
  static final long serialVersionUID = -7030051827420762568L;

  /**
   * Default constructor.
   */
  CTMPoint()
  {
	super();
  }

  /**
   * Constructor.
   *
   * @param aX The x component of the point.
   * @param aY The y component of the point.
   */
  CTMPoint( int aX, int aY )
  {
	super( aX, aY );
  }

  /**
   * Constructor.
   *
   * @param aPoint The point to duplicate.
   */
  CTMPoint( Point aPoint )
  {
	super( aPoint );
  }

  /**
   * Create a new instance of ctmpoint that is halfway between aPoint1 and
   * aPoint2.
   *
   * @param aPoint1 First point for midpoint.
   * @param aPoint2 Second point for midpoint.
   * @return The new midpoint instance.
   */
  public static CTMPoint getMidpointInstance( Point aPoint1, Point aPoint2 )
  {
	return new CTMPoint( ( aPoint1.x+aPoint2.x )/2,
		( aPoint1.y+aPoint2.y )/2 );
  }

  /**
   * Extends to x and y extents of the point to which ever has bigger extents,
   * aMaxTest or this.
   *
   * @param aMaxTest The point to extend to if need be.
   */
  void resizeMax( Point aMaxTest )
  {
	if ( x<aMaxTest.x )
	{
	  x = aMaxTest.x;
	}
	if ( y<aMaxTest.y )
	{
	  y = aMaxTest.y;
	}
  }

  /**
   * Add aPoint to this.
   *
   * @param aPoint The point to translate by.
   * @return this
   */
  CTMPoint add( Point2D aPoint )
  {
	x += aPoint.getX();
	y += aPoint.getY();
	return this;
  }

  /**
   * Add aPoint to this.
   *
   * @param aX The horizontal amount to translate by.
   * @param aY The vertical amount to translate by.
   * @return this
   */
  CTMPoint add( int aX, int aY )
  {
	x += aX;
	y += aY;
	return this;
  }

  /**
   * Subtract aPoint from this.
   *
   * @param aSubPoint The point to translate by.
   * @return this
   */
  CTMPoint subtract( Point aSubPoint )
  {
	x -= aSubPoint.x;
	y -= aSubPoint.y;
	return this;
  }

  /**
   * Subtract aPoint from this.
   *
   * @param aX The horizontal amount to translate by.
   * @param aY The vertical amount to translate by.
   * @return this
   */
  CTMPoint subtract( int aX, int aY )
  {
	x -= aX;
	y -= aY;
	return this;
  }

  /**
   * Multiple x and y by aScale.
   *
   * @param aScale double Multiple x and y by aScale.
   * @return CTMPoint
   */
  CTMPoint scale( double aScale )
  {
	x *= aScale;
	y *= aScale;

	return this;
  }

  /**
   * Set point values.
   *
   * @param aPoint Point The new values to assign to this.
   * @return CTMPoint
   */
  CTMPoint set( Point aPoint )
  {
	x = aPoint.x;
	y = aPoint.y;

	return this;
  }

  /**
   * Translate an array of points.
   *
   * @param aPoints Point[] Array of points.
   * @param aX X translation.
   * @param aY Y translation.
   */
  static void translate( Point[] aPoints, int aX, int aY )
  {
	for ( int i = 0; i<aPoints.length; i++ )
	{
	  aPoints[i].translate( aX, aY );
	}
  }
}
